/**************************************************************************
 **************************************************************************
 
  Program:  Gstep_TWI_Functions		Date: Version: 0.92
   
   Author:  SS	  					  Date:  June 11, 2010
   
  Compiled with ImageCraft ICCAVR Standard V7.30.
  
  Copyright 2002-2009 SOC Robotics, Inc.
 
  TWI Functions for handling peripherals such as Motors, DRO's, joysticks
  etc.
		 
 ***************************************************************************	 
 ***************************************************************************/
#include <iom644v.h>
#include <macros.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <eeprom.h>
#include <stdio.h>

//#include "uart.h"
#include "Wasp_Uart.h"
#include "Wasp_Defines.h"
#include "Wasp_Extern.h" 

// ***********************************************************************
// ***********************************************************************
//     Start of Code
// ***********************************************************************
// ***********************************************************************


#define	   TWI_MTX_ADR_NACK  0x20

void Init_ADS1100_TWI(unsigned char twiaddress)
{
//unsigned char ADS1100writeAdd;
	//ADS1100writeAdd = (twiaddress << 1 | 0x00);
 	 messageBuf[0] = 0x0c;
	 TWI_Start_Write(ADS1100ADDRESS,messageBuf,0);
}

void Init_BMP085_TWI(unsigned char twiaddress)
{
 	 messageBuf[0] = 0x0c;
	 TWI_Start_Write(twiaddress,messageBuf,0);
}

/*
void Write_ADS1100_Register(unsigned char twiaddress,unsigned char outbyte)
{
    messageBuf[0] = outbyte;
    TWI_Start_Write(twiaddress,messageBuf,1);
}
*/
unsigned char Read_ADS1100_Register(unsigned char twiaddress,unsigned char iregister)
{	//unsigned char ADS1100readAdd;
	//ADS1100readAdd = (twiaddress << 1 | 0x01);
	TWI_Start_Read(ADS1100ADDRESS,iregister);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

// Configure SmartLCD display device via TWI to accept float
void  Init_SmartLCD_TWI(unsigned char twiaddress)
{
	 messageBuf[0] = ESC;
	 messageBuf[1] = 'c';
	 TWI_Start_Write(twiaddress,messageBuf,2);
}

// Start a measurement event in the Digital Compass
void  Init_DigitalCompass_TWI()
{
	 messageBuf[0] = 'W';  // Exit sleep state
	 TWI_Start_Write(DIGITALCOMPASSTWIADDRESS,messageBuf,1);
}

void Write_DigitalCompass_Register(unsigned char twiaddress,unsigned char outbyte)
{
    messageBuf[0] = outbyte;
    TWI_Start_Write(twiaddress,messageBuf,1);
}

unsigned char Read_DigitalCompass_Register(unsigned char twiaddress,unsigned char iregister)
{
	TWI_Start_Read(twiaddress,iregister);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

unsigned char Read_HMC5883_Register(unsigned char iregister)
{
	messageBuf[0] = iregister;
	TWI_Start_Write(HMC5883TWIADDRESS,messageBuf,1);
	TWI_Start_Read(HMC5883TWIADDRESS,1);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

void Write_HMC5883_Register(unsigned char iregister,unsigned char data)
{
	messageBuf[0] = iregister;
	messageBuf[1] = data;
	TWI_Start_Write(HMC5883TWIADDRESS,messageBuf,2);
}

void Init_HMC5883_TWI()
{
	messageBuf[0] = HMC5883_MODE_REG;
	messageBuf[1] = HMC5883_CONTINUOUS_M;  // Set continuous measurement mode
	TWI_Start_Write(HMC5883TWIADDRESS,messageBuf,2);
}

unsigned char flipByte(unsigned char byteToFlip)
{		unsigned char flippedByte; 

		unsigned char flippedByteCopy; 
		flippedByte = 0x00;
		
		flippedByteCopy = byteToFlip;
		flippedByte |= (0b00001000 & (flippedByteCopy >> 1 ));
		flippedByte |= (0b00000100 & (flippedByteCopy >> 2 ));
		flippedByte |= (0b00000010 & (flippedByteCopy >> 2 ));
		flippedByte |= (0b00000001 & (flippedByteCopy >> 2 ));
		
		 
		flippedByteCopy = byteToFlip;
		flippedByte |= (0b00010000 & (flippedByteCopy << 1 ));
		flippedByte |= (0b00100000 & (flippedByteCopy << 2 ));
		flippedByte |= (0b01000000 & (flippedByteCopy << 2 ));
		flippedByte |= (0b10000000 & (flippedByteCopy << 2 ));
		 
		return flippedByte; 
}

unsigned int Read_ADS1100()
{		
 	unsigned char ADSreading;
	unsigned int returnValue;
	union {
	    unsigned char tempbuf[2];
		int outint;
		} u;
	
	u.outint = 0;
	TWI_Start_Read(ADS1100ADDRESS,3);
    TWI_Get_Data_From_Transceiver(messageBuf,3);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	returnValue = u.outint ;//+ 32768;
//	printf("\nmessageBuff0: %x\n\r", messageBuf[0]);
//	printf(" messageBuff1: %x\n\r", messageBuf[1]);
//	printf("messageBuff2: %x\n\r", messageBuf[2]);
    return returnValue;
}

unsigned char Read_BMP085_EEPROM(unsigned char eepromaddress)
{
	messageBuf[0] = eepromaddress;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	TWI_Start_Read(BMP085TWIADDRESS,1);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
	return messageBuf[0];
}

int Read_BMP085_EEPROM_Signed(unsigned char eepromaddress)
{
	union {
	    unsigned char tempbuf[2];
		int outint;
		} u;
	
	u.outint = 0;
	u.tempbuf[1] = Read_BMP085_EEPROM(eepromaddress++);
	u.tempbuf[0] = Read_BMP085_EEPROM(eepromaddress);
    return u.outint;  
}

unsigned int Read_BMP085_EEPROM_UnSigned(unsigned char eepromaddress)
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
	u.tempbuf[1] = Read_BMP085_EEPROM(eepromaddress++);
	u.tempbuf[0] = Read_BMP085_EEPROM(eepromaddress);
    return u.outint;  
}

void Load_BMP085_Parameters()
{
 	 bmpac1=bmpac2=bmpac3=bmpb1=bmpb2=bmpmb=bmpmc=bmpmd=0;
	 bmpac4=bmpac5=bmpac6=0;

 	 // Read signed BMP085 Parameters
	 bmpac1 = Read_BMP085_EEPROM_Signed(0xaa);
	 bmpac2 = Read_BMP085_EEPROM_Signed(0xac);
	 bmpac3 = Read_BMP085_EEPROM_Signed(0xae);
	 bmpb1 = Read_BMP085_EEPROM_Signed(0xb6);
	 bmpb2 = Read_BMP085_EEPROM_Signed(0xb8);
	 bmpmb = Read_BMP085_EEPROM_Signed(0xba);
	 bmpmc = Read_BMP085_EEPROM_Signed(0xbc);
	 bmpmd = Read_BMP085_EEPROM_Signed(0xbe);
	 if(0) {
	     printf("\n\rBMP085 Calibration Parameters\n\r");
	     printf("ac1=%d\n\r",bmpac1);
	     printf("ac2=%d\n\r",bmpac2);
	     printf("ac3=%d\n\r",bmpac3);
	     printf("bp1=%d\n\r",bmpb1);
	     printf("bp2=%d\n\r",bmpb2);
	     printf("mb=%d\n\r",bmpmb);
	     printf("mc=%d\n\r",bmpmc);
	     printf("md=%d\n\r",bmpmd);
		 }
	 
	 // Read unsigned BMP085 parameters
	 bmpac4 = Read_BMP085_EEPROM_UnSigned(0xb0);
	 bmpac5 = Read_BMP085_EEPROM_UnSigned(0xb2);
	 bmpac6 = Read_BMP085_EEPROM_UnSigned(0xb4);
	 if(0) {
	     printf("ac4=%u\n\r",bmpac4);
	     printf("ac5=%u\n\r",bmpac5);
	     printf("ac6=%u\n\r",bmpac6);
		 }
}

unsigned int Read_BMP085_Pressure()
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
    //printf("\n\rRead barometric pressure\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0xb4;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,2);
	Delaymsec(28);
	messageBuf[0] = 0xf6;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP085TWIADDRESS,2);
    TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	//printf("mb[0]= %x\n\r",messageBuf[0]);
	//printf("mb[1]= %x\n\r",messageBuf[1]);
	
    return u.outint;
}

unsigned int Read_BMP085_Temperature()
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
    //printf("\n\rRead temperature\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0x2e;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,2);
	Delaymsec(6);
	messageBuf[0] = 0xf6;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP085TWIADDRESS,2);
    TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	//printf("mb[0]= %x\n\r",messageBuf[0]);
	//printf("mb[1]= %x\n\r",messageBuf[1]);
	
    return u.outint;
}
unsigned int Read_DigitalCompass_Heading()
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
	Write_DigitalCompass_Register(DIGITALCOMPASSTWIADDRESS,'A');
    //printf("\n\rRead digital compass\n\r");
	TWI_Start_Read(DIGITALCOMPASSTWIADDRESS,3);
    TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	//printf("messagebug[0]=%x\n\r",messageBuf[0]);
	//printf("messagebug[1]=%x\n\r",messageBuf[1]);
	
    return u.outint;
}

// Write float value to LED8 via TWI
void Write_Float_SmartLCD(unsigned char twiaddress,unsigned char axisid,float fvalue)
{
 	 void *value;
	 
	 messageBuf[0] = 'm';
	 messageBuf[1] = axisid;
	 value = &messageBuf[2];
	 *(float *)value = fvalue;
	 TWI_Start_Write(twiaddress,messageBuf,6);
}

unsigned char Read_SmartLCD_Switches(char twiaddress)
{
	TWI_Start_Read(twiaddress,3);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

void putcharSmartLCD(char inchar)
{	 
	 messageBuf[0] = inchar;
	 TWI_Start_Write(SMARTLCDTWIADDRESS,messageBuf,1);
}

void putsSmartLCD(char *strg)
{
 	unsigned char i;
	
 	for(i=0; i<strlen(strg); i++) {
	    putcharSmartLCD(strg[i]);
		Delaymsec(5);
		}
}

void Display_SmartLCD()
{
    float xpos,ypos,zpos,apos;
	
	//xpos = ((float)xoldpos)*xresolution;
	//if(xdirpolarity==NEGATIVE)  xpos = -xpos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'x',xpos);
	Delaymsec(4);
	
	//ypos = ((float)yoldpos)*yresolution;
	//if(ydirpolarity==NEGATIVE)  ypos = -ypos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'y',ypos);
	Delaymsec(4);

	//zpos = ((float)zoldpos)*zresolution;
	//if(zdirpolarity==NEGATIVE)  zpos = -zpos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'z',zpos);
	Delaymsec(4);
					  
	//apos = ((float)holdpos)*aresolution;
	//if(adirpolarity==NEGATIVE)  apos = -apos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'a',apos);
	//delaymsec(5);
}

long int Read_caliper()
{	
	unsigned char caliperReading;
	long int returnValue;
	unsigned char returnCode;
	union {
	    unsigned char tempbuf[4];
		long int outint;
		} u2;
		
		
	//reading digital caliper
	u2.outint = 0;
	u2.tempbuf[0]=0;
	u2.tempbuf[1]=0;
	u2.tempbuf[2]=0;
	u2.tempbuf[3]=0;
	
	
	while ((PIND & (1<< DDD7))==0 );
	
	returnCode = TWI_Start_Read(ATTINY85ADDRESS,4);
	//printf("return code: %x\n\r", returnCode );
		//Delaymsec(100);
		
	if (returnCode ==0)
	{	
		TWI_Get_Data_From_Transceiver(messageBuf,4);
	   	u2.tempbuf[0]=messageBuf[0];
		u2.tempbuf[1]=messageBuf[1];
		u2.tempbuf[2]=messageBuf[2];
		u2.tempbuf[3]=0;
	   	//Delaymsec(100);
		// printf(" caliper0: %x\n\r", u2.tempbuf[0]);
		// printf(" caliper1: %x\n\r", u2.tempbuf[1]);
		//printf(" caliper2: %x\n\r", u2.tempbuf[2]);
	}
	
	returnValue = u2.outint ;
	
	return returnValue;
}

void Enumerate_ADS()
{
 	 
	  ADSstate = 0;
	 // Check for ADS
	 Init_ADS1100_TWI(ADS1100ADDRESS);
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	ADSstate |= 0x01;
	 	 Delaymsec(7);
		 }
}

void Enumerate_BMP085()
{
	 BMPstate = bmpac1 = 0;
	 // Check for ADS
	 bmpac1 = Read_BMP085_EEPROM_Signed(0xaa);
	 //printf("bmpac1 = %d\n\r",bmpac1);
     if(bmpac1!=0) {
	 	 BMPstate |= 0x01;
	 	 Delaymsec(7);
		 }
}

void Enumerate_SmartLCD()
{
 	 smartlcdstate = 0;
	 
	 // Check for Smart Joystick
	 Init_SmartLCD_TWI(SMARTLCDTWIADDRESS);
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 smartlcdstate |= 0x01;
	 	 Delaymsec(50);
	 	 //Display_SmartLCD();
		 }
}

void Enumerate_DigitalCompass()
{
 	 digitalcompassstate = 0;
	 
	 // Send A command to digital compass to start a conversion
	 //printf("Init compass\n\r");
	 Init_DigitalCompass_TWI();
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 digitalcompassstate |= 0x01;
		 Read_DigitalCompass_Heading();  // read first wrong value
	 	 Delaymsec(7);
		 }
}

void Enumerate_HMC5883()
{
 	 hmc5883state = 0;
	 
	 // Send A command to digital compass to start a conversion
	 //printf("Init compass\n\r");
	 Init_HMC5883_TWI();
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 hmc5883state |= 0x01;
		 //Read_DigitalCompass_Heading();  // read first wrong value
	 	 Delaymsec(7);
		 }
}

const char enummes1[] = "\n\rEnumerate TWI Devices\n\r";
const char enummes8[] = " No SmartLCD found\n\r";
const char enummes9[] = " SmartLCD found - enabled\n\r";
const char enummes10[] = " No Digital Compass\n\r";
const char enummes11[] = " Digital Compass found\n\r";
const char enummes12[] = " No ADS1100\n\r";
const char enummes13[] = " ADS1100 found\n\r";
const char enummes14[] = " No BMP085 1 found\n\r";
const char enummes15[] = " BMP085 1 found\n\r";
const char enummes16[] = " No BMP085 2 found\n\r";
const char enummes17[] = " BMP085 2 found\n\r";
const char enummes18[] = " No HMC5883 found\n\r";
const char enummes19[] = " HMC5883 found\n\r";

void Enumerate_TWI_Devices()
{
 	 unsigned char i,motormask=0x01;
	
	 putfs(enummes1);	
	 
	 // Enumerate ADS1100
	 Enumerate_ADS();
	 if(ADSstate==0) putfs(enummes12);
	 else putfs(enummes13);
	 
	 // Enumerate BMP085 Sensor 1
//	 EnableBMP085(BMPSENSOR1);
	 Enumerate_BMP085();
	 if(BMPstate==0) putfs(enummes14);
	 else putfs(enummes15);
//	 EnableBMP085(BMPSENSOR2);
//	 BMPstate = 0;
//	 Enumerate_BMP085();
//	 if(BMPstate==0) putfs(enummes16);
//	 else putfs(enummes17);
	 
	 // Enumerate SmartLCD
	 Enumerate_SmartLCD();
	 if(smartlcdstate==0) putfs(enummes8);
	 else putfs(enummes9);
	 
	 // Enumerate Digital Compass
	 Enumerate_DigitalCompass();
	 if(digitalcompassstate==0) putfs(enummes10);
	 else putfs(enummes11);
	 
	 Enumerate_HMC5883();
	 if(hmc5883state==0) putfs(enummes18);
	 else putfs(enummes19);
}

const char TWImess1[] = "\n\rTWI Interactive Command Mode\n\r";
const char TWImess2[] = "Enter twi address (hex-two chars) followed by ASCII commands\n\r";
const char TWImess3[] = "  Example:  14xtx    Enter 'e' to exit\n\r";

// Send commands on I2C bus given twi address and an ASCII string
//  TBD - get data command
void Execute_TWI_Command()
{ 
     unsigned char twiaddress,iptr;
	  
	 putfs(TWImess1);
	 putfs(TWImess2);
	 putfs(TWImess3);
	 putchar(':');
 	 while((instrg[0]=tolower(getcharr()))!='e') {
	     instrg[1] = getcharr();
		 instrg[2] = 0;
		 sscanf(instrg,"%x",&twiaddress);
		 iptr = 0;
	     while((messageBuf[iptr++]=getcharr())!=CR) {
		     if(iptr==20) break;
			 }
		 messageBuf[iptr] = 0;
		 
		 printf("\n\rtwi=%x %s",twiaddress,messageBuf);
		 
		 TWI_Start_Write(twiaddress,messageBuf,iptr);

	     putCRLF(); putchar(':');
	     }
	 putCRLF();
}

